package com.unleashedsoftware.sdk.example;

import com.google.inject.*;
import com.unleashedsoftware.sdk.model.*;
import com.unleashedsoftware.sdk.network.*;
import com.unleashedsoftware.sdk.security.*;
import org.codehaus.jackson.map.*;
import java.io.*;

import static java.lang.System.out;

public class SalesOrderSave {
    public UnleashedApiCredentials credentials;

    private UnleashedApiAuthSignatureCalculator calculator;
    private RestClient client;
    private RestClientReadWriteRequest request;
    private ObjectMapper serialiser;

    @Inject
    public SalesOrderSave(UnleashedApiAuthSignatureCalculator calculator, RestClient client) {
        this.calculator = calculator;
        this.client = client;
        this.serialiser = new ObjectMapper();
        this.serialiser.disable(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES);
    }

    public void save(ApiSalesOrder salesOrder) throws Exception {
        init(salesOrder);
        try {
            tryPost();
            handleSuccessPost();
        } catch (PostException error) {
            handleNotSuccessPost(error);
        } catch (Exception error) {
            handleUnexpectedError(error);
        }
    }

    private void init(ApiSalesOrder salesOrder) throws Exception {
        String url = String.format("https://api.unleashedsoftware.com/SalesOrders/%s", salesOrder.id);
        String query = "";
        calculateSignatureUsingQueryString(query);
        createRequest(url, query, salesOrder);
    }

    private void calculateSignatureUsingQueryString(String query) throws Exception {
        credentials.hash = query;
        calculator.calculate(credentials);
    }

    private void createRequest(String url, String query, ApiSalesOrder salesOrder) throws IOException {
        request = new RestClientReadWriteRequest();
        request.requestUrl = query.length() > 0 ? String.format("%s?%s", url, query) : url;
        request.requestBody = serialiser.writeValueAsString(salesOrder);
        request.headers.addHeader("User-Agent", "Unleashed");
        request.headers.addHeader("Content-Type", "application/json");
        request.headers.addHeader("Content-Length", Integer.toString(request.requestBody.getBytes().length));
        request.headers.addHeader("Accept", "application/json");
        request.headers.addHeader("api-auth-id", credentials.authId);
        request.headers.addHeader("api-auth-signature", credentials.authSignature);
        request.connectionTimeoutInMilliseconds = 60000;
    }


    private void tryPost() throws IOException {
        client.post(request);
        if(request.hasError())
            throw new PostException();
    }

    private void handleSuccessPost() throws IOException {
        ApiSalesOrder salesOrder = serialiser.readValue(request.requestResult, ApiSalesOrder.class);
        out.println(String.format("Id:%s, OrderNumber:%s", salesOrder.id, salesOrder.orderNumber));
    }

    private void handleNotSuccessPost(PostException error) throws IOException {
        if(request.requestResultHttpCode > 400) handleRequestError();
        else handleBadRequestError();
    }

    private void handleRequestError() throws IOException {
        ApiRequestError requestError = serialiser.readValue(request.requestResult, ApiRequestError.class);
        System.out.println(String.format("Description:%s", requestError.description));
        System.out.println(String.format("DebugInformation:%s", requestError.debugInformation));
    }

    private void handleBadRequestError() throws IOException {
        ApiSubmitErrorResult submitErrorResult = serialiser.readValue(request.requestResult, ApiSubmitErrorResult.class);
        for(ApiSubmitError e : submitErrorResult.errors)
            out.println(String.format("Field:%s, Description:%s", e.field, e.description));
    }

    private void handleUnexpectedError(Exception error) {
        throw new UnsupportedOperationException();
    }

    public class PostException extends RuntimeException {
    }
}
